use super::*;

pub type IParseResult<'a, Ctx, F, I> = Result<(Mtbl<'a, Ctx, F>, I), ParseError<'a, Ctx, F>>;

pub trait CInliningFactory<'a, Ctx: Context<'a>>:
    FactoryBase<'a, Ctx> + ImplMode<Mode = InliningMode>
{
    fn cextension_error(&self, tail: &[u8]) -> Self::ParseError;

    fn cideserialize<I: InCtx<'a, Ctx>>(&self, inctx: I) -> IParseResult<'a, Ctx, Self, I>;
}

/// This factory should return an error on EOF.
pub trait InliningFactory<'a, Ctx: Context<'a>>:
    FactoryBase<'a, Ctx> + ParseMode<Mode = InliningMode>
{
    fn extension_error(&self, tail: &[u8]) -> Self::ParseError;

    fn ideserialize<I: InCtx<'a, Ctx>>(&self, inctx: I) -> IParseResult<'a, Ctx, Self, I>;
}

impl<'a, Ctx: Context<'a>, F: FactoryModeParse<'a, Ctx> + ParseMode<Mode = InliningMode>>
    InliningFactory<'a, Ctx> for F
{
    fn extension_error(&self, tail: &[u8]) -> Self::ParseError {
        self.mextend((), tail)
    }

    fn ideserialize<I: InCtx<'a, Ctx>>(&self, inctx: I) -> IParseResult<'a, Ctx, Self, I> {
        self.mdeserialize(inctx)
    }
}

impl<'a, Ctx: Context<'a>, F: CInliningFactory<'a, Ctx>> FactoryModeProxy<'a, Ctx>
    for WithMode<F, InliningMode>
{
    type F = F;

    fn pmdeserialize<I: InCtx<'a, Ctx>>(f: &Self::F, inctx: I) -> ModeResultM<'a, Ctx, F, I> {
        f.cideserialize(inctx)
    }

    fn pmextend(
        f: &F,
        _mentionable: ExtensionSourceM<'a, Ctx, F>,
        tail: &[u8],
    ) -> ExtensionResultM<'a, Ctx, F> {
        f.cextension_error(tail)
    }
}
